﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Entities
{
    using System.Collections.Generic;

    using EdiFabric.Framework.Constants;

    internal class ParseTree : IEqualityComparer<ParseTree>
    {
        public string Name { get; set; }

        public string TypeName { get; set; }

        public string AssemblyName { get; set; }

        public ParseTree Parent { get; set; }

        public List<ParseTree> Children { get; set; }

        public string Prefix
        {
            get
            {
                return this.Name.Split(new[] { '_' })[0];
            }
        }

        public string EdiName
        {
            get
            {
                return this.Name.Split(new[] { '_' })[1];
            }
        }

        public bool IsMessage
        {
            get
            {
                return this.Prefix == EdiPrefix.M;
            }
        }

        public bool IsGroup
        {
            get
            {
                return this.Prefix == EdiPrefix.G;
            }
        }

        public bool IsSegment
        {
            get
            {
                return this.Prefix == EdiPrefix.S;
            }
        }

        public bool IsComplex
        {
            get
            {
                return this.Prefix == EdiPrefix.C;
            }
        }

        public bool IsSimple
        {
            get
            {
                return this.Prefix == EdiPrefix.D;
            }
        }

        public bool IsTrigger
        {
            get
            {
                if (this.Parent != null && this.Parent.Prefix == EdiPrefix.G)
                {
                    return this.Parent.Children.IndexOf(this) == 0;
                }

                return false;
            }
        }

        public bool Equals(ParseTree x, ParseTree y)
        {
            return x.EdiName == y.EdiName;
        }

        public int GetHashCode(ParseTree obj)
        {
            return obj.EdiName.GetHashCode();
        }
    }
}
