﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Edi
{
    using System;
    using System.Linq;
    using System.Xml.Linq;

    using EdiFabric.Framework.Constants;
    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Exceptions;
    using EdiFabric.Framework.Helpers;
    using EdiFabric.Framework.InterchangeEnvelopes.Edifact;

    internal class EdiDirector
    {
        private EdiBuilder builder;

        private readonly XElement xmlEdi;

        private readonly InterchangeContext interchangeContext;

        public EdiDirector(XElement xmlEdi, InterchangeContext interchangeContext)
        {
            this.xmlEdi = xmlEdi;
            this.interchangeContext = interchangeContext;
        }

        public void Construct(EdiBuilder ediBuilder, string origin)
        {
            if (ediBuilder == null) throw new ArgumentNullException("ediBuilder");

            this.builder = ediBuilder;
            XNamespace ns = xmlEdi.Name.NamespaceName;

            this.builder.CreateInterchangeSettings(xmlEdi);
            this.builder.CreateInterchangeHeader(xmlEdi);

            var groups = this.xmlEdi.Element(ns + Internals.Groups);
            if (groups == null)
            {
                throw new ParserException("Can't find GROUPS element.");
            }

            if (!groups.Elements().Any())
            {
                throw new ParserException("Can't find GROUP element.");
            }

            foreach (var group in groups.Elements())
            {
                this.builder.CreateGroupHeader(group);

                var messages = group.Element(ns + Internals.Messages);

                if (messages == null)
                {
                    throw new ParserException("Can't find MESSAGES element.");
                }

                if (!messages.Elements().Any())
                {
                    throw new ParserException("Can't find MESSAGE element.");
                }

                foreach (var message in messages.Elements())
                {
                    var typedMessage = message.Elements().FirstOrDefault();

                    if (typedMessage == null)
                    {
                        throw new ParserException("Can't find message.");
                    }

                    if (!typedMessage.Elements().Any())
                    {
                        throw new ParserException("Can't find segments.");
                    }

                    foreach (var segment in typedMessage.Elements())
                    {
                        this.builder.CreateSegment(segment, typedMessage, origin);
                    }
                }

                this.builder.CreateGroupTrailer(group);
            }

            this.builder.CreateInterchangeTrailer(xmlEdi);
        }
    }
}
