﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Xml.Linq;

    using EdiFabric.Framework.Constants;
    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Exceptions;
    using EdiFabric.Framework.Helpers;
    using EdiFabric.Framework.Parsers.Interchange;
    using EdiFabric.Framework.Parsers.Edi;

    public class EdiParser : IEdiParser
    {
        public T ToInterchange<T>(Stream ediStream, string origin)
        {
            InterchangeEdiBuilder builder;
            var interchangeContext = InterchangeContextHelper.GetInterchangeContext(EdiHelper.CopyStream(ediStream));
            var director = new InterchangeDirector(ediStream, interchangeContext, origin);

            switch (interchangeContext.Format)
            {
                case Format.Edifact:
                    builder = new InterchangeEdifactBuilder(interchangeContext);
                    break;
                case Format.X12:
                    builder = new InterchangeX12Builder(interchangeContext);
                    break;
                default:
                    throw new ParserException("Unsupported format " + interchangeContext.Format);
            }

            director.Construct(builder);

            return (T)builder.InterchangeEnvelope;
        }

        public List<string> ToEdi<T>(T interchangeEnvelope, string origin, InterchangeContext context)
        {
            EdiBuilder builder;
            var xmlEdi = Serialize(interchangeEnvelope);
            var interchangeContext = InterchangeContextHelper.MergeInterchangeContext(xmlEdi, context);
            var director = new EdiDirector(xmlEdi, interchangeContext);

            switch (interchangeContext.Format)
            {
                case Format.Edifact:
                    builder = new EdifactBuilder(interchangeContext);
                    break;
                case Format.X12:
                    builder = new X12Builder(interchangeContext);
                    break;
                default:
                    throw new ParserException("Unsupported format " + interchangeContext.Format);
            }

            director.Construct(builder, origin);

            return builder.EdiSegments;
        }

        public XElement Serialize<T>(T instance)
        {
            return EdiHelper.Serialize(instance);
        }

        public T Deserialize<T>(XElement xml)
        {
            return EdiHelper.Deserialize<T>(xml);
        }
    }
}
