﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Interchange
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;

    using EdiFabric.Framework.Constants;
    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Helpers;

    class InterchangeDirector
    {
        private InterchangeEdiBuilder builder;

        private readonly InterchangeContext interchangeContext;

        private readonly string origin;

        private readonly string[] ediSegments;

        public InterchangeDirector(Stream ediStream, InterchangeContext interchangeContext, string origin)
        {
            if (ediStream == null) throw new ArgumentNullException("ediStream");
            if (interchangeContext == null) throw new ArgumentNullException("interchangeContext");

            this.origin = origin;
            this.interchangeContext = interchangeContext;
            using (ediStream)
            {
                this.ediSegments = EdiHelper.GetEdiSegments(EdiHelper.CopyStream(ediStream), interchangeContext);
            }
        }

        public void Construct(InterchangeEdiBuilder ediBuilder)
        {
            if (ediBuilder == null) throw new ArgumentNullException("ediBuilder");

            this.builder = ediBuilder;
            var message = new List<string>();
            var header = new List<string>();
            
            foreach (var segment in this.ediSegments)
            {
                var segmentName = EdiHelper.GetSegmentName(segment, this.interchangeContext);
                switch (segmentName)
                {
                    case Segment.Una:
                        break;
                    case Segment.Unb:
                    case Segment.Isa:
                        this.builder.CreateInterchangeHeader(segment);
                        break;
                    case Segment.Unz:
                    case Segment.Iea:
                        this.builder.CreateInterchangeTrailer(segment);
                        break;
                    case Segment.Ung:
                        this.builder.CreateGroupHeader(segment);
                        break;
                    case Segment.Gs:
                        this.builder.CreateGroupHeader(segment);
                        header.Add(segment);
                        break;
                    case Segment.Une:
                    case Segment.Ge:
                        this.builder.CreateGroupTrailer(segment);
                        header.Clear();
                        break;
                    case Segment.Unh:
                    case Segment.St:
                        message.Add(segment);
                        header.Add(segment);
                        break;
                    case Segment.Unt:
                    case Segment.Se:
                        message.Add(segment);
                        var messageContext = this.builder.GetMessageContext(header, this.origin);
                        header.Remove(header.Last());
                        this.builder.CreateMessage(message, messageContext, this.origin);
                        message.Clear();
                        break;
                    default:
                        message.Add(segment);
                        break;
                }
            }
        }
    }
}
