﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Interchange
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml.Linq;

    using EdiFabric.Framework.Constants;
    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Exceptions;
    using EdiFabric.Framework.Helpers;
    using EdiFabric.Framework.InterchangeEnvelopes.Edifact;
    using EdiFabric.Framework.Parsers.Message;

    class InterchangeEdifactBuilder : InterchangeEdiBuilder
    {
        private INTERCHANGE EdifactEnvelope
        {
            get
            {
                return this.interchangeEnvelope as INTERCHANGE;
            }
        }

        public InterchangeEdifactBuilder(InterchangeContext interchangeContext)
        {
            if (interchangeContext == null) throw new ArgumentNullException("interchangeContext");

            this.interchangeContext = interchangeContext;
            this.interchangeEnvelope = new INTERCHANGE { GROUPS = new List<GROUP>() };
        }

        public override void CreateInterchangeHeader(string segment)
        {
            EdifactEnvelope.S_UNB = ParserHelper.PopulateSegment<S_UNB>(segment, this.interchangeContext);
        }

        public override void CreateInterchangeTrailer(string segment)
        {
            EdifactEnvelope.S_UNZ = ParserHelper.PopulateSegment<S_UNZ>(segment, this.interchangeContext);
        }

        public override void CreateGroupHeader(string segment)
        {
            this.EdifactEnvelope.GROUPS.Add(new GROUP());
            this.EdifactEnvelope.GROUPS.Last().MESSAGES = new List<XElement>();
            this.EdifactEnvelope.GROUPS.Last().S_UNG = ParserHelper.PopulateSegment<S_UNG>(segment, this.interchangeContext);
        }

        public override void CreateGroupTrailer(string segment)
        {
            this.EdifactEnvelope.GROUPS.Last().S_UNE = ParserHelper.PopulateSegment<S_UNE>(segment, this.interchangeContext);
        }

        public override void CreateMessage(List<string> segments, MessageContext messageContext, string origin)
        {
            if (segments == null) throw new ArgumentNullException("segments");

            if (this.EdifactEnvelope.GROUPS.Count == 0)
            {
                this.EdifactEnvelope.GROUPS.Add(new GROUP());
                this.EdifactEnvelope.GROUPS.Last().MESSAGES = new List<XElement>();
            }

            var parseTree = ParserHelper.GetParsingGrammarForMessage(messageContext);
            var director = new MessageDirector(segments, this.interchangeContext);
            var xmlEdi = director.Construct(parseTree);

            this.EdifactEnvelope.GROUPS.Last().MESSAGES.Add(xmlEdi);
        }

        public override MessageContext GetMessageContext(IEnumerable<string> ediSegments, string origin)
        {
            if (ediSegments == null) throw new ArgumentNullException("ediSegments");
            
            try
            {
                var unh = ediSegments.Single(es => es.StartsWith(Segment.Unh));
                var ediCompositeDataElements = EdiHelper.GetEdiCompositeDataElements(unh, this.interchangeContext);
                var ediDataElements = EdiHelper.GetEdiComponentDataElements(ediCompositeDataElements[1], this.interchangeContext);

                var result = new MessageContext
                {
                    Tag = ediDataElements[0],
                    Version = ediDataElements[1] + ediDataElements[2],
                    Origin = origin,
                    Format = Format.Edifact
                };

                return result;
            }
            catch
            {
                throw new ParserException("Can't parse UNH segment.");
            }
        }
    }
}
