﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Message
{
    using System;
    using System.Xml.Linq;

    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Helpers;

    internal class MessageBuilder
    {
        public XElement EdiMessage { get; set; }

        private XElement lastProcessed;

        private XElement lastParent;

        private readonly InterchangeContext interchangeContext;

        public MessageBuilder(InterchangeContext interchangeContext)
        {
            if (interchangeContext == null) throw new ArgumentNullException("interchangeContext");

            this.interchangeContext = interchangeContext;
        }

        public void CreateRoot(string messageName)
        {
            if (string.IsNullOrEmpty(messageName)) throw new Exception("Message name is blank.");

            XNamespace ns = this.interchangeContext.TargetNamespace;
            this.EdiMessage = new XElement(ns + messageName);
            this.SetPosition(null, this.EdiMessage);
        }

        public void CreateElement(ParsingContext parsingContext)
        {
            if (parsingContext == null) throw new ArgumentNullException("parsingContext");

            var parent = this.GetNextParent(parsingContext);

            XElement segment;
            if (parsingContext.ContextEntity.IsSegment)
            {
                parsingContext.ContextEntity = ParserHelper.GetParsingGrammarForSegment(parsingContext.ContextEntity);
                segment = ParserHelper.ParseSegment(parsingContext, this.interchangeContext);
            }
            else
            {
                XNamespace ns = interchangeContext.TargetNamespace;
                segment = new XElement(ns + parsingContext.ContextEntity.Name);
            }

            parent.Add(segment);
            this.SetPosition(parent, segment);
        }

        private XElement GetNextParent(ParsingContext intermediateEdi)
        {
            //  Parent is message
            if (this.lastParent == null) return this.lastProcessed;

            if (this.lastParent == null) throw new Exception("Last Parent is null");
            if (this.lastProcessed == null) throw new Exception("Last Processed is null");

            //  Parent is the same level
            if (intermediateEdi.ContextEntity.Parent.Name == this.lastParent.Name.LocalName) return this.lastParent;
            //  Parent is a level down
            return intermediateEdi.ContextEntity.Parent.Name == this.lastProcessed.Name.LocalName
                       ? this.lastProcessed
                       : this.GetPreviousParent(intermediateEdi.ContextEntity.Parent.Name, this.lastParent);
        }

        private XElement GetPreviousParent(string name, XElement entity)
        {
            //  Parent is message
            if (entity.Parent == null) return null;

            //  Parent is a level up or continue searching in the upper levels
            return entity.Parent.Name.LocalName == name ? entity.Parent : this.GetPreviousParent(name, entity.Parent);
        }

        private void SetPosition(XElement parent, XElement current)
        {
            this.lastParent = parent;
            this.lastProcessed = current;
        }
    }
}
