﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Message
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml.Linq;

    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Exceptions;

    class MessageDirector
    {
        private readonly InterchangeContext interchangeContext;
        private readonly List<string> ediSegments;

        public MessageDirector(List<string> segments, InterchangeContext interchangeContext)
        {
            if (segments == null) throw new ArgumentNullException("segments");
            if (interchangeContext == null) throw new ArgumentNullException("interchangeContext");

            this.ediSegments = segments;
            this.interchangeContext = interchangeContext;
        }

        public XElement Construct(ParseTree parsingGrammar)
        {
            if (parsingGrammar == null) throw new ArgumentNullException("parsingGrammar");

            return this.ToXml(this.ToParsingContext(parsingGrammar));
        }

        private List<ParsingContext> ToParsingContext(ParseTree ediParseTree)
        {
            if (ediParseTree == null) throw new ArgumentNullException("ediParseTree");

            var parsingContextBuilder = new ParsingContextBuilder(ediParseTree, this.interchangeContext);
            string currentLine = "No line processed";

            try
            {
                foreach (var segment in this.ediSegments)
                {
                    currentLine = segment;
                    parsingContextBuilder.CreateParsingContextItem(segment);
                }
            }
            catch (Exception ex)
            {
                throw new ParserException("Last processed line was: " + currentLine, ex);
            }

            return parsingContextBuilder.ParsingContext;
        }

        private XElement ToXml(List<ParsingContext> parsingContext)
        {
            if (parsingContext == null) throw new ArgumentNullException("parsingContext");
            if (parsingContext.First() == null) throw new Exception("parsingContext has no elements");
            if (parsingContext.First().ContextEntity == null) throw new Exception("parsingContext has no context entity");
            if (string.IsNullOrEmpty(parsingContext.First().ContextLine)) throw new Exception("parsingContext has no context line");
            if (parsingContext.First().ContextEntity.Parent == null) throw new Exception("parsingContext has no message");

            var messageBuilder = new MessageBuilder(this.interchangeContext);
            string currentLine = parsingContext.First().ContextLine;

            try
            {
                messageBuilder.CreateRoot(parsingContext.First().ContextEntity.Parent.Name);
                
                foreach (var context in parsingContext)
                {
                    currentLine = context.ContextLine;
                    messageBuilder.CreateElement(context);
                }
            }
            catch (Exception ex)
            {
                throw new ParserException("Last processed line was: " + currentLine, ex);
            }

            return messageBuilder.EdiMessage;
        }
    }
}
