﻿//---------------------------------------------------------------------
// This file is part of ediFabric
//
// Copyright (c) ediFabric. All rights reserved.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//---------------------------------------------------------------------

namespace EdiFabric.Framework.Parsers.Message
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using EdiFabric.Framework.Entities;
    using EdiFabric.Framework.Helpers;

    class ParsingContextBuilder
    {
        public List<ParsingContext> ParsingContext { get; set; }
        private ParseTree lastProcessed;
        private readonly ParseTree contextGrammar;
        private readonly InterchangeContext interchangeContext;

        public ParsingContextBuilder(ParseTree ediParseTree, InterchangeContext interchangeContext)
        {
            if (ediParseTree == null) throw new ArgumentNullException("ediParseTree");
            if (interchangeContext == null) throw new ArgumentNullException("interchangeContext");

            this.ParsingContext = new List<ParsingContext>();
            this.contextGrammar = ediParseTree;
            this.interchangeContext = interchangeContext;
            this.lastProcessed = this.contextGrammar.Children.First();
        }

        public void CreateParsingContextItem(string ediSegment)
        {
            if (String.IsNullOrEmpty(ediSegment)) throw new ArgumentNullException("ediSegment");

            this.lastProcessed = this.GetNextMatch(ediSegment);
            
            if (this.lastProcessed.IsTrigger)
            {
                var groupContext = new ParsingContext { ContextLine = null, ContextEntity = this.lastProcessed.Parent };
                this.ParsingContext.Add(groupContext);
            }

            var segmentContext = new ParsingContext { ContextLine = ediSegment, ContextEntity = this.lastProcessed };
            this.ParsingContext.Add(segmentContext);
        }

        private ParseTree GetNextMatch(string ediSegment)
        {
            if (String.IsNullOrEmpty(ediSegment)) throw new ArgumentNullException("ediSegment");

            var ediSegmentName = EdiHelper.GetSegmentName(ediSegment, this.interchangeContext);

            var indexOfLastProcessed = this.lastProcessed.Parent.Children.IndexOf(this.lastProcessed);
            var foundSegment = this.FindOnSameLevel(ediSegmentName, this.lastProcessed.Parent.Children, indexOfLastProcessed);
            if (foundSegment != null) return foundSegment;

            foundSegment = this.FindOnUpperLevel(ediSegmentName, this.lastProcessed.Parent);
            return foundSegment;
        }

        private ParseTree FindOnSameLevel(string ediSegmentName, IEnumerable<ParseTree> segments, int index)
        {
            var segment = segments.Skip(index).FirstOrDefault(s => s.EdiName == ediSegmentName);
            if (segment != null && segment.IsGroup) return segment.Children.First();
            return segment;
        }

        private ParseTree FindOnUpperLevel(string ediSegmentName, ParseTree entity)
        {
            var indexOfLastProcessed = entity.Parent.Children.IndexOf(entity);
            var result = this.FindOnSameLevel(ediSegmentName, entity.Parent.Children, indexOfLastProcessed);
            return result ?? this.FindOnUpperLevel(ediSegmentName, entity.Parent);
        }
    }
}
